import React, { Component } from 'react';
import App from './App';
import reducers from './reducers/reducer';
import { createStore,combineReducers } from 'redux';
import { Provider } from 'react-redux';
import {NativeEventEmitter} from 'react-native';
import JitsiManager from './lib/JitsiManager';
import Logger from './lib/Logger';
import AudioMode from './AudioMode'
import CallingComponent from 'react-native-cometchat-calling-component/CallingComponent';
import RTCManager from 'react-native-cometchat-calling-component/lib/RTCManager';
import {NativeModules} from 'react-native';
const { CometChatRTC } = NativeModules;

var eventEmitter = new NativeEventEmitter(AudioMode);

export default class CometChatCallingComponent extends Component{
    constructor(props){
        super(props);
        const rootReducer = combineReducers(reducers);
        this.storeObj = createStore(rootReducer);
        JitsiManager.setCallObject(this.props.RTCUser,this.props.RTCReceiver,this.props.RTCInitiator,this.props.isInitiator);
        JitsiManager.setRegion(this.props.REGION);
        JitsiManager.setAudioOnly(this.props.isAudioOnly);
    }
    
    componentDidMount() {
        this.eventListener = new NativeEventEmitter(AudioMode);
        this.eventListener.addListener('CallEvent',(type) =>{
            
            if(type.type == "onUserJoinned"){
                Logger.log("CallEvent onUserJoinned : ",type.uid)
                this.props.onUserJoinned(type.uid);
            }

            if(type.type == "onUserLeft"){
                Logger.log("CallEvent onUserLeft : ",type.uid)
                this.props.onUserLeft(type.uid);
            }

            if(type.type == "EndCall"){
                this.props.onEndCall();
            }

            if(type.type == "EndCallButtonPressed"){
                this.props.onCallEndButtonPressed();
            }
        });
    }

    render(){
        return(
            <Provider store={this.storeObj}>
                <App 
                    sessionid={this.props.sessionId}
                    enableDefaultLayout = {this.props.enableDefaultLayout}
                    hideAudioMuteButton = {this.props.hideAudioMuteButton}
                    hideEndCallButton = {this.props.hideEndCallButton}
                    hidePauseVideoButton = {this.props.hidePauseVideoButton}
                    hideVideoSwitchButton = {this.props.hideVideoSwitchButton}
                    hideAudioModeButton = {this.props.hideAudioModeButton}
                    isAudioOnly = {this.props.isAudioOnly}
                    isConference = {this.props.isConference}
                    uid = {this.props.uid}
                    RTCUser = {this.props.RTCUser}
                    isSingleMode = {this.props.isSingleMode}
                    mode = {this.props.mode}
                    REGION = {this.props.REGION}
                />
            </Provider>
        );
    }
}  
CometChatCallingComponent.endCallSession=()=>{
    CometChatRTC.log("123 endCallSession")
    RTCManager.endCall();
}

CometChatCallingComponent.switchCameraSource=()=>{
    RTCManager.switchCamera();
}
CometChatCallingComponent.muteAudio=()=>{
    RTCManager.muteAudio();
}
CometChatCallingComponent.unMuteAudio=()=>{
    RTCManager.unMuteAudio();
}
CometChatCallingComponent.pauseVideo=()=>{
    RTCManager.pauseVideo()
}
CometChatCallingComponent.unPauseVideo=()=>{
    RTCManager.unPauseVideo()
}
CometChatCallingComponent.setAudioModeToSpeaker=()=>{
    //JitsiManager.setAudioModeSpeaker();
}
CometChatCallingComponent.setAudioModeToEarPiece=()=>{
    //JitsiManager.setAudioModeEarpiece();
}